%%  Author: Panos N. Alevizos
%
%   Ph.D. Student and Research Assistant
%   School of Electrical and Computer Engineering
%   Technical University of Crete
%   Email: palevizos@isc.tuc.gr or bigpan88@hotmail.com
%   Webpage: http://users.isc.tuc.gr/~palevizos/
%
%   Date: February 2017
%
%   References:
%   P. N. Alevizos, A. Bletsas, G. N. Karystinos, "Noncoherent Short Packet
%   Detection and Decoding for Scatter Radio Sensor Networking," IEEE Transactions On Communications,
%   Volume: XX, Issue: XX, XX, pp. XX--XX, 2017.
%
%   **Inquiries regarding the script provided below are cordially welcome.
%   In case you spot a bug, please let me know.
%   If you use some piece of code for your own work, please cite the
%   article above.**


%% ---
%% this code implements GLRT sequence detection of Section III.B with the 
%% baseband signal model of Eq.~(32) 
%% ---

clear all;
close all;
clc;


%%  communication parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Coherence time in symbol periods
N_coh = 100;

%number of info bits within coherence time interval
N_info = N_coh;

%number of total bits within coherence time interval
N_tot = N_info ;

%sequence length factor
sf = 1; %sf=2 %sf=5, %sf=10, %sf_50, %sf=50, %sf=100
%sequence detection length
N_seq = N_tot/sf;


%average energy per bit (assuming rate = 1)
E_b = 1;


%transmit SNR for tag
SNR_dB = 0 : 2 : 20;
%transmit SNR for tag (scenario 3)
%  SNR_dB = [[0 : 2 : 14], 15 , 16];

%different rician scenarios
% % % Rice K factor for each link (scenario 1)
% K_CT = 0; % Rician k factor for emitter-> tag link
% K_TR = 0;%; % Rician k factor for tag-> reader link

% % % Rice K factor for each link (scenario 2)
K_CT = 20; % Rician k factor for emitter-> tag link
K_TR = 10;%; % Rician k factor for tag-> reader link

% % Rice K factor for each link (scenario 3)
% K_CT = 1000000000000;     % Rician k factor for emitter-> tag link
% K_TR = 1000000000000;     % Rician k factor for tag-> reader link



%symbols
s0 = [1;1;0;0];
s1 = [0;0;1;1];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%   BER parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%total ber vector fro aproximate ML for simulations
BER_NC_sequence_simul_uncoded = zeros(length(N_seq), length(SNR_dB));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%  simulation parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% number of itertations
N_iter = 5*[1000, 1200, 1800, 2500, 4200, 6600, 8500, 10000 , 12000, 15000, 19000, 25000];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%construc search phi
Phi_check_points = 0 : pi/3 : 2*pi - pi/3;
len_Phi_check_points = length(Phi_check_points);

%possible phase configurations for Phi_0 kai Phi_1
Phi0_check = kron(Phi_check_points, ones(1,len_Phi_check_points));
Phi1_check = kron(ones(1,len_Phi_check_points), Phi_check_points);

%GLRT-optimal sequence and GLRT metric value for each grid check point
b_GLRT_phi_check = zeros(round(len_Phi_check_points^2), N_tot);
val_GLRT_phi_check = zeros(round(len_Phi_check_points^2), 1);

%GLRT optimal sequence
b_GLRT = zeros(1, N_tot);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%                          MONTE CARLO
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for snr_index = 1 : length(SNR_dB) %snr iterations
    
    %calculation of current noise variance
    N0_2 =  (E_b) / (10^(SNR_dB(snr_index)/10)) 
    
    for ii = 1 : N_iter(snr_index) %iteration index

%         ii
        %info bit sequence (scattered from tag)
        b_info =  sign(randn(1, N_tot))*.5+.5;
    
        
        %complex flat channel  from emmiter to tag
        h_CT = (sqrt(K_CT /(K_CT + 1)) + sqrt(1/(2*(K_CT + 1))) * (randn + 1i*randn)); %ricean fading
        
        %complex flat channel from tag to reader
        h_TR = (sqrt(K_TR /(K_TR + 1)) + sqrt(1/(2*(K_TR + 1))) * (randn + 1i*randn)); %ricean fading
        
        %ricean amplitudes
        a_CT = abs(h_CT);
        a_TR = abs(h_TR);
        
        %random phases
        Delta_phi = 2*pi*rand;              % emitter-reader LO phase
        phi_CT = angle(h_CT);               % emitter->tag channel
        phi_TR = angle(h_TR);               % tag->reader channel
        
        %total phases (W.L. OF G. assume angle(Gamma_0 - Gamma_1) = 0)
        phi = Delta_phi + phi_CT + phi_TR;
        
        %random initial phase due to tag-reader phase mismatch
        Phi_0 =  (2*pi*rand);
        Phi_1 =  (2*pi*rand);
        
        %the random phases of received symbols
        exp_Phi_random = [exp(1i*Phi_0); exp(-1i*Phi_0); exp(1i*Phi_1); exp(-1i*Phi_1)];
      
        %transmitted symbols for coherent and noncoherent
        s_NC = sqrt(E_b/2)*(kron(b_info,s1.*exp_Phi_random) + kron(1-b_info , s0.*exp_Phi_random));
    
        
        %compound channel
        h = a_CT*a_TR*exp(-1i*phi);
        
        % CN noise
        noise = sqrt(N0_2/2)*randn(size(s_NC)) + 1i*sqrt(N0_2/2)*randn(size(s_NC));
        
        % received signal for noncoherent
        y_FSK_NC = h*s_NC + noise;
        
        
        %%  noncoherent sequence detection approach (Algorithm 2)
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        for jj = 1 : N_tot/N_seq

            for kk = 1 : round(length(Phi_check_points)^2)
            y_GLRT = [y_FSK_NC(1, (jj-1)*N_seq + 1: jj*N_seq)*exp(-1i * Phi0_check(kk)) + y_FSK_NC(2, (jj-1)*N_seq + 1: jj*N_seq)*exp(1i * Phi0_check(kk));
                      y_FSK_NC(3, (jj-1)*N_seq + 1: jj*N_seq)*exp(-1i * Phi1_check(kk)) + y_FSK_NC(4, (jj-1)*N_seq + 1: jj*N_seq)*exp(1i * Phi1_check(kk))];

             %apply algorithm 1
            [ b_GLRT_cur_phi_check, GLRT_val_cur_phi_check ] = BistaticBFSKNonoherentSequenceDetection( y_GLRT );
            %
            
            b_GLRT_phi_check(kk, (jj-1)*N_seq + 1: jj*N_seq) = b_GLRT_cur_phi_check;
            val_GLRT_phi_check(kk) = GLRT_val_cur_phi_check;
            end
            
            %GLRT optimal sequence (for current phase configuration)
            [~, ind_GLRT_opt] = max(val_GLRT_phi_check);
            b_GLRT((jj-1)*N_seq + 1: jj*N_seq) = b_GLRT_phi_check(ind_GLRT_opt,(jj-1)*N_seq + 1: jj*N_seq);
            %accumulate BER
            BER_NC_sequence_simul_uncoded(snr_index) = BER_NC_sequence_simul_uncoded(snr_index) +...
                sum(b_GLRT((jj-1)*N_seq + 1: jj*N_seq)~=b_info( (jj-1)*N_seq + 1: jj*N_seq));
        end
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
    end %end iter

    %compute the simulated overall bit error rate of the GLRT sequence
    %detector
    BER_NC_sequence_simul_uncoded(:, snr_index) = BER_NC_sequence_simul_uncoded(:, snr_index) / (N_tot*N_iter(snr_index));
    
end %end snr iteration
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 
