%%  Author: Panos N. Alevizos
%
%   Ph.D. Student and Research Assistant
%   School of Electrical and Computer Engineering
%   Technical University of Crete
%   Email: palevizos@isc.tuc.gr or bigpan88@hotmail.com
%   Webpage: http://users.isc.tuc.gr/~palevizos/
%
%   Date: February 2017
%
%   References:
%   P. N. Alevizos, A. Bletsas, G. N. Karystinos, "Noncoherent Short Packet
%   Detection and Decoding for Scatter Radio Sensor Networking," IEEE Transactions On Communications,
%   Volume: XX, Issue: XX, XX, pp. XX--XX, 2017.
%
%   **Inquiries regarding the script provided below are cordially welcome.
%   In case you spot a bug, please let me know.
%   If you use some piece of code for your own work, please cite the
%   article above.**

%% ---
%% this code implements noncoherent HCHT decodig of Section III.C with the 
%% baseband signal model of Eq.~(49) 
%% ---

clear all;
close all;
clc;



%%                    channel coding parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%parameters for BCH
n = 31; k = 11; d_interl = 1; %create generator matrix corresponding to int. depth = 1
[ n, k , ~, d_min, rate, G, Code, Code_Symbols ]  = BCH( n, k, d_interl );
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%              parameters for interleaving
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%number of total bits within coherence time interval
N_coh = 100;

%temporary variables for block interleaving
c_tmp = zeros(1,N_coh);


%the size of actual transmitted bit within the coherence time
length_transmission = N_coh;

%redundant symbols length
length_redundant = 0;

%interleaver depth
d_interl = 100;

%how many times the size of the whole sequence is larger than the coherence
%time
numCodeLengthPerCohTime = ceil(d_interl *n / N_coh);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




%%  communication parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%number of info bits for noncoherent system
N_info = N_coh;


% %total ammount of usefull info bits per total transmitted bits
rate_NC = (d_interl*k) / (d_interl*n);

%average energy per bit (assuming rate = 1)
E_b = 1;
%average energy per coded bit for NC
E_coded_NC = E_b*rate_NC;


%transmit SNR for tag
SNR_dB = [0:2:20];
% SNR_dB = [[0:2:10] ,11,12];
% SNR_dB = 20

%different rician scenarios
% % Rice K factor for each link (scenario 1)
% K_CT = 0; % Rician k factor for emitter-> tag link
% K_TR = 0;%; % Rician k factor for tag-> reader link

% % Rice K factor for each link (scenario 2)
K_CT = 20; % Rician k factor for emitter-> tag link
K_TR = 10;%.3; % Rician k factor for tag-> reader link

% % Rice K factor for each link (scenario 3)
% K_CT = 1000000000000;     % Rician k factor for emitter-> tag link
% K_TR = 1000000000000;%.3; % Rician k factor for tag-> reader link


%symbols
s0 = [1;1;0;0];
s1 = [0;0;1;1];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%   BER parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
BER_NC_simul_coded_BCH = zeros(length(K_CT), length(SNR_dB));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%  simulation parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% number of itertations
 N_iter = 25*[1000, 1200, 1800, 4500, 8200, 10600, 13500, 16000 , 20000, 25000, 31000];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%








%%                     MONTE CARLO phase
for snr_index = 1 : length(SNR_dB) %snr iterations
    
    %calculation of current noise variance
    N0_2 =  (E_b) / (10^(SNR_dB(snr_index)/10))
    
    
    %intermidiate variable that stores the observations of the
    %correlators for the whole block
    y_interl = zeros(1,d_interl*n);
    
    for ii = 1 : N_iter(snr_index) %iteration index
        
        %             ii
        
        %information sequence
        b_info =  sign(randn(d_interl, k))*.5+.5;
        
        %coded sequence
        c_info = mod(b_info * G,2);
        
        %interleaved sequence (read column-wise)
        c_transmit = c_info(1:end);
        
        %random initial phase due to tag-reader phase mismatch
        Phi_0 = (2*pi*rand);
        Phi_1 =  (2*pi*rand);
        
        for jj = 1 : numCodeLengthPerCohTime %for each coherence time block
            %                 jj
            
            %if the whole block containts interleaved sequence
            if(jj < numCodeLengthPerCohTime)
                c_tmp = c_transmit((jj-1)*N_info + 1 : jj*N_info);
                %the number of remaining coded bits in the packet
                length_transmission = N_info;
                %the number of redundant bits
                length_redundant = 0;
            else %at the endof packet, if the rest  interleaved bits duariation
                %is smaller than coherence time interval we pefrom zero padding
                c_tmp = [ c_transmit((jj-1)*N_info + 1 : end), zeros(1,jj*N_info - length(c_transmit))];
                %the number of remaining coded bits in the packet
                length_transmission = length(c_transmit((jj-1)*N_info + 1 : end));
                %the number of redundant bits
                length_redundant = N_info - length_transmission;
            end
            
            
            %complex flat channel  from emmiter to tag
            h_CT = (sqrt(K_CT /(K_CT + 1)) + sqrt(1/(2*(K_CT + 1))) * (randn + 1i*randn)); %ricean fading
            
            %complex flat channel from tag to reader
            h_TR = (sqrt(K_TR /(K_TR + 1)) + sqrt(1/(2*(K_TR + 1))) * (randn + 1i*randn)); %ricean fading
            
            %ricean amplitudes
            a_CT = abs(h_CT);
            a_TR = abs(h_TR);
            
            %random phases
            Delta_phi = 2*pi*rand;       % emitter-reader LO phase
            phi_CT = angle(h_CT);               % emitter->tag channel
            phi_TR = angle(h_TR);               % tag->reader channel
            
            %total phases (W.L. OF G. assume angle(Gamma_0 - Gamma_1) = 0)
            phi = Delta_phi + phi_CT + phi_TR;
            
            
            %the random phases of received symbols
            exp_Phi_random = [exp(1i*Phi_0); exp(-1i*Phi_0); exp(1i*Phi_1); exp(-1i*Phi_1)];
            
            %transmitted symbols for coherent and noncoherent
            s_NC = sqrt(E_coded_NC/2)*(kron(c_tmp,s1.*exp_Phi_random) + kron(1-c_tmp , s0.*exp_Phi_random));
            
            %compound channel
            h = a_CT*a_TR*exp(-1i*phi);
            
            % CN noise
            noise = sqrt(N0_2/2)*randn(size(s_NC)) + 1i*sqrt(N0_2/2)*randn(size(s_NC));
            
            % received signal for coherent and noncoherent
            y_FSK_NC = h*s_NC + noise;
            
            
            %%  baseband equivalent signal
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %noncoherent soft-decision metric for HCHT
            soft_decisions =  abs(y_FSK_NC(3,1:end)).^2 + abs(y_FSK_NC(4,1:end)).^2 ...
                - (abs(y_FSK_NC(1,1:end)).^2 + abs(y_FSK_NC(2,1:end)).^2) ;
            
            
            %%
            %each coherence time transmission (correlator outputs) are mapped to a
            %piece of a whole codeword - length_transmission at the
            %right hand side is mandatory since the whole codeword
            %length is not multiple of coherence time
            y_interl(1, N_info*(jj-1) +1 : N_info*jj -  length_redundant ) = soft_decisions(1 : length_transmission );
            
        end
        
        %% Noncoherent HCHT decoding
        %%%%%%%%%%%%%%%%%%%%%%%%%%%
        for ll = 1 : d_interl
            
            %metric of maximization
            soft_dec = (y_interl( ll : d_interl : end )  * Code');
            
            %find the index that maximizes the inner product
            [~,ind1] = max(soft_dec);
            
            %take the word with the biggest index
            bits_est_FSK_NC = numberToString(ind1 - 1,2,k);
            
            %acumulate ber
            BER_NC_simul_coded_BCH(snr_index) = BER_NC_simul_coded_BCH( snr_index) +...
                mean(bits_est_FSK_NC ~= b_info(ll,:));
            
        end
        %%%%%%%%%%%%%%%%%%%%%%%%%%%
    end %end experiments
    
    %%  Coded non-coherent
    %compute the overall bit error rate 
    BER_NC_simul_coded_BCH( snr_index) = BER_NC_simul_coded_BCH(snr_index)...
        /  (N_iter(snr_index)*d_interl)
    
end %end snr iteration
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
