%%  Author: Panos N. Alevizos
%
%   Ph.D. Student and Research Assistant
%   School of Electrical and Computer Engineering
%   Technical University of Crete
%   Email: palevizos@isc.tuc.gr or bigpan88@hotmail.com
%   Webpage: http://users.isc.tuc.gr/~palevizos/
%
%   Date: February 2017
%
%   References:
%   P. N. Alevizos, A. Bletsas, G. N. Karystinos, "Noncoherent Short Packet
%   Detection and Decoding for Scatter Radio Sensor Networking," IEEE Transactions On Communications,
%   Volume: XX, Issue: XX, XX, pp. XX--XX, 2017.
%
%   **Inquiries regarding the script provided below are cordially welcome.
%   In case you spot a bug, please let me know.
%   If you use some piece of code for your own work, please cite the
%   article above.**

%% ---
%% this code implements the signal processing receiving chain for the 
%% system model of Section II and  HCHT symbol-by-symbol detection
%% ---


clear all;
close all;
clc;


%%  communication parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% bit period
T = 1e-3;

% sampling period
Ts = T/10;

%carrier emitter power (at passband)
P_carrier = 20*10^(-3);

%scattering efficiency
scattering_eff = 10^(-2);

%antenna structural mode (the value is take from bIstatic Kimionis paper [2])
A_s = 0.6047 + 1i*0.5042;

%Gamma_0, Gamma_1 (imperfect load values)
Gamma_0 = 0.95 + 0.04*1i;
Gamma_1 = -0.97 + 0.02*1i;
Delta_Gamma = Gamma_0 - Gamma_1;

%oversampling factor
L = round(T/Ts);

%coherence time interval (normalized with bit period) 
N = 100;

%defining the time axis (for a single symbol)
t_symbol = 0:Ts:T-Ts;

%total transmited time axis
t_total = 0: Ts : N*T - Ts;

%subfrequencies for FSK (orthogonality criterion must be sustisfied)
F0 = 1500;
F1 = F0 + 4*(1/T);

% precompute basis functions for correlation
basis_0_pos = exp(-1i*2*pi*F0*t_total);
basis_0_neg = exp(+1i*2*pi*F0*t_total);
basis_1_pos = exp(-1i*2*pi*F1*t_total);
basis_1_neg = exp(+1i*2*pi*F1*t_total);

%transmit SNR for tag
SNR_dB = 0:5:30;
SNR = 10.^(SNR_dB/10)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%   BER parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%total ber vector for uncoded system
BER_SL_simul_uncoded = zeros(1, length(SNR_dB));
BER_SL_theory_uncoded = zeros(1, length(SNR_dB));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%   Simulation parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% number of itertations
N_iter = [1000, 2200, 2800, 4500, 6200, 8600, 10500, 12000 , 15000];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%                          Monte Carlo
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for snr_index = 1 : length(SNR_dB) %snr iterations
    
    %calculation of current variance (N0_2 = N_0/2)
    N0_2 =   (T * scattering_eff^2 * 8 * P_carrier * abs(Delta_Gamma)^2) / (pi^2 * 10^(SNR_dB(snr_index)/10)) %
    
    for ii = 1 : N_iter(snr_index) %iteration index

%         ii
        
        %info bits
        b =  sign(randn(1, N))*.5+.5;
        
        %random phases (valid only for Rayleigh fading)
        Delta_phi = 2*pi*rand;            % emitter-reader LO phase
        phi_CR =  2*pi*rand;              % emitter->reader channel
        phi_CT = 2*pi*rand;               % emitter->tag channel
        phi_TR =2*pi*rand;                % tag->reader channel
        
        %total phases of  Eq. (7)
        phi_DC = phi_CR + Delta_phi;
        phi = Delta_phi + phi_CT + phi_TR;
        
        %CFO
        DF = 30;
        
        %emmiter to reader  (valid only for Rayleigh fading)
        %channel from emmiter to reader
        a_CR = abs(sqrt(0.5) * (randn + 1i*randn)); %rayleygh fading
        
        %emmiter to tag
        %channel from emmiter to tag
        a_CT = abs(sqrt(0.5) * (randn + 1i*randn)); %rayleygh fading
        
        %tag to reader
        %channel from tag to reader
        a_TR = abs(sqrt(0.5) * (randn + 1i*randn)); %rayleygh fading
        
        %random initial phase due to tag-reader phase mismatch
        Phi_0 = (2*pi*rand);
        Phi_1 = (2*pi*rand);
        
        % random initial phase
        Phi_random_0 = (Phi_0)*kron(1-b, ones(1,L));
        Phi_random_1 =  (Phi_1)*kron(b, ones(1,L));
        
        %backscattered signal (50% duty cycle square pulse)
        u_FSK_bit0 = (4/pi)*(Delta_Gamma/2)*kron(1-b, ones(1,L)).*cos(2*pi*F0*t_total + Phi_random_0);
        u_FSK_bit1 = (4/pi)*(Delta_Gamma/2)*kron(b, ones(1,L)).*cos(2*pi*F1*t_total + Phi_random_1);
        u_FSK = scattering_eff*((A_s - (Gamma_0 + Gamma_1)/2) + u_FSK_bit0 + u_FSK_bit1);
           
        %FSK baseband equivalent
        x_FSK = sqrt(2*P_carrier)*((a_CR*exp(-1i*phi_DC) + a_CT*a_TR*exp(-1i*(phi))) + ...
            a_CT*a_TR*exp(-1i*(phi))*u_FSK);
        
        % CN noise (variane per complex baseband sample sigma^2 = N0_2 / (2 Ts))
        noise = sqrt(N0_2/(4*Ts))*randn(size(x_FSK)) + 1i*sqrt(N0_2/(4*Ts))*randn(size(x_FSK));
        
        % received signal with CFO
        y_nl_FSK =  (x_FSK).*exp(-1i*2*pi*DF*t_total) ;
        y_FSK = y_nl_FSK + noise;
        
        
        %% perfect synchronization
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        
        %%  CFO estimation
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %assume perfect CFO
        DF_est = -DF;
        %compensate CFO
        y_FSK = y_FSK.*exp(-1i*2*pi*DF_est*t_total);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
        
        
        %%  DC blocking 
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        y_FSK = y_FSK - mean(y_FSK);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        
        %%  matched filtering
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %multiply the received signal with the corresponding basis
        %functions
        mul0_pos = y_FSK.*basis_0_pos;
        mul0_neg = y_FSK.*basis_0_neg;
        mul1_pos = y_FSK.*basis_1_pos;
        mul1_neg = y_FSK.*basis_1_neg;
        
        %integrators
        r0_pos = Ts*sum(reshape(mul0_pos, L, N)) ;
        r0_neg = Ts*sum(reshape(mul0_neg, L, N)) ;
        r1_pos = Ts*sum(reshape(mul1_pos, L, N));
        r1_neg = Ts*sum(reshape(mul1_neg, L, N)) ;
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %%   Non-Coherent HCHT symbol-by-symbol detection
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %square-law detection
        bits_est_FSK_NC =(abs(r1_pos).^2+ abs(r1_neg).^2) > (abs(r0_pos).^2+ abs(r0_neg).^2) ;
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %accumulate the error
        BER_SL_simul_uncoded(snr_index) = BER_SL_simul_uncoded(snr_index) + mean(bits_est_FSK_NC~=b);
        
    end %end iter
    
    %ber calculation in theory (Rayleigh fading)
    BER_SL_theory_uncoded(snr_index) = -(SNR(snr_index) +...
        ( exp(2/SNR(snr_index))*(5*SNR(snr_index) + 2) *(real(-expint(2/SNR(snr_index))))))/(4*SNR(snr_index)^2)
    
    %Normalize the BERS
    BER_SL_simul_uncoded(snr_index) = BER_SL_simul_uncoded(snr_index) / (N_iter(snr_index));
    
end %end snr iteration
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%





